package installment_payment

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	enum2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/installment_payment/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/installment_payment/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/implement"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

// AddInstallmentPaymentList 新增 "分期付-订单" 记录
func AddInstallmentPaymentList(engine *xorm.Engine, masterId string, args md.AddInstallmentPaymentListStruct) (err error) {
	now := time.Now()
	//1、查找对应方案记录
	installmentPaymentSchemeDb := implement.NewInstallmentPaymentSchemeDb(engine)
	scheme, err := installmentPaymentSchemeDb.GetInstallmentPaymentSchemeById(args.SchemeId)
	if err != nil {
		return err
	}
	if scheme == nil {
		return fmt.Errorf("installmentPaymentSchemeDb.InstallmentPaymentSchemeById returned nil")
	}

	//2、计算相关数据
	installmentNumsValue := decimal.NewFromInt32(int32(scheme.InstallmentNums))     //分期数
	totalAmount, _ := decimal.NewFromString(args.TotalAmount)                       //总金额
	repaymentAmountPerInstallment := totalAmount.Div(installmentNumsValue).String() //每期还款金额

	//3、新增 installment_payment_list 记录
	installmentPaymentListDb := implement.NewInstallmentPaymentListDb(engine)
	_, err = installmentPaymentListDb.InsertInstallmentPaymentList(&models.InstallmentPaymentList{
		Uid:                           args.Uid,
		GoodsId:                       args.GoodsId,
		OrdId:                         args.OrdId,
		TotalAmount:                   args.TotalAmount,
		InstallmentNums:               scheme.InstallmentNums,
		AlreadyRepaidInstallmentNums:  0,
		RepaymentAmountPerInstallment: repaymentAmountPerInstallment,
		IsRepaidOff:                   0,
		CreateAt:                      now.Format("2006-01-02 15:04:05"),
		UpdateAt:                      now.Format("2006-01-02 15:04:05"),
	})
	if err != nil {
		return err
	}
	return
}

// InstallmentPaymentManualRepaid 手动还款
func InstallmentPaymentManualRepaid(engine *xorm.Engine, masterId string, args md.InstallmentPaymentManualRepaid) (err error) {
	now := time.Now()
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	//1、查找对应`installment_payment_list`记录
	installmentPaymentListDb := implement.NewInstallmentPaymentListDb(engine)
	installmentPaymentList, err := installmentPaymentListDb.GetInstallmentPaymentListById(args.RecordId)
	if err != nil {
		session.Rollback()
		return err
	}
	if installmentPaymentList == nil {
		session.Rollback()
		return fmt.Errorf("installmentPaymentListDb.InstallmentPaymentListById returned nil")
	}

	//2、计算相关数据
	repaidAmount, _ := decimal.NewFromString(args.RepaidAmount)                                                           //还款总金额
	repaymentAmountPerInstallmentValue, _ := decimal.NewFromString(installmentPaymentList.RepaymentAmountPerInstallment)  //每期还款金额
	installmentNumsValue := decimal.NewFromInt32(int32(installmentPaymentList.InstallmentNums))                           //分期数
	alreadyRepaidInstallmentNumsValue := decimal.NewFromInt32(int32(installmentPaymentList.AlreadyRepaidInstallmentNums)) //已还分期数
	residueRepaidInstallmentNumsValue := installmentNumsValue.Sub(alreadyRepaidInstallmentNumsValue)                      //剩余分期数
	residueRepaidTotalAmountValue := residueRepaidInstallmentNumsValue.Mul(repaymentAmountPerInstallmentValue)            //剩余待还总金额
	if args.RepaidKind == 1 {
		//判断还款金额是否正确
		if installmentPaymentList.RepaymentAmountPerInstallment != args.RepaidAmount {
			session.Rollback()
			return errors.New("还款金额与按期还款金额不一致")
		}
		//按期还款
		installmentPaymentList.AlreadyRepaidInstallmentNums++
		if installmentPaymentList.AlreadyRepaidInstallmentNums == installmentPaymentList.InstallmentNums {
			installmentPaymentList.IsRepaidOff = int(enum2.InstallmentPaymentListRepaidOn) //TODO::已还完
		}
	} else {
		//判断还款金额是否正确
		if repaidAmount.GreaterThan(residueRepaidTotalAmountValue) {
			session.Rollback()
			return errors.New("还款金额大于剩余待还总金额")
		}
		if repaidAmount.LessThan(residueRepaidTotalAmountValue) {
			session.Rollback()
			return errors.New("还款金额小于剩余待还总金额")
		}
		//全额还款
		installmentPaymentList.AlreadyRepaidInstallmentNums++
		installmentPaymentList.IsRepaidOff = int(enum2.InstallmentPaymentListRepaidOn) //TODO::已还完
	}

	//3、处理用户余额
	orderType := enum.FinUserFlowOrderActionString(md2.InstallmentPaymentAutoRepaidForFinUserFlow)
	var dealUserAmount = md2.DealUserAmount{
		Kind:      "sub",
		Mid:       masterId,
		Title:     md2.InstallmentPaymentAutoRepaidTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md2.InstallmentPaymentAutoRepaidForFinUserFlow,
		OrdId:     zhios_order_relate_utils.Int64ToStr(installmentPaymentList.OrdId),
		Uid:       args.Uid,
		Amount:    zhios_order_relate_utils.StrToFloat64(args.RepaidAmount),
	}
	err = svc.DealUserAmount(session, dealUserAmount)
	if err != nil {
		session.Rollback()
		return err
	}

	//4、新增 installment_payment_repaid_flow 记录
	installmentPaymentRepaidFlowDb := implement.NewInstallmentPaymentRepaidFlowDb(engine)
	_, err = installmentPaymentRepaidFlowDb.InsertInstallmentPaymentRepaidFlowBySession(session, &models.InstallmentPaymentRepaidFlow{
		RecordId:             installmentPaymentList.Id,
		Uid:                  installmentPaymentList.Uid,
		Amount:               installmentPaymentList.RepaymentAmountPerInstallment,
		WhichRepaymentPeriod: installmentPaymentList.AlreadyRepaidInstallmentNums,
		CreateAt:             now.Format("2006-01-02 15:04:05"),
		UpdateAt:             now.Format("2006-01-02 15:04:05"),
	})
	if err != nil {
		session.Rollback()
		return err
	}

	//5、修改`installment_payment_list` 记录
	updateAffected, err := installmentPaymentListDb.UpdateInstallmentPaymentListBySess(session, installmentPaymentList, "already_repaid_installment_nums", "is_repaid_off")
	if err != nil {
		session.Rollback()
		return err
	}
	if updateAffected <= 0 {
		session.Rollback()
		return errors.New("更新 `installment_payment_list` 记录失败")
	}
	err = session.Commit()
	if err != nil {
		session.Rollback()
		return err
	}
	return
}
